import { Session } from "./session.ts";
import { KeyListener, Term } from "./term.ts";

export interface Input {
    getChar(): Promise<string>;
}

export class KeyListenerInput implements Input {
    public constructor(
        private keyListener: KeyListener,
    ) {}

    getChar(): Promise<string> {
        return new Promise((resolve) => {
            const id = setInterval(() => {
                if (this.keyListener.hasPressedKey()) {
                    const key = this.keyListener.nextPress();
                    resolve(key);
                    clearInterval(id);
                    return;
                }
            }, 10);
        });
    }
}

export type IO = {
    input: Input;
    term: Term;
    session: Session;
};

export interface Program {
    run(io: IO, args: string[]): Promise<number>;
}
